Package: flowSpecs
Version: 1.24.0
Date: 2023-04-05
Type: Package
Title: Tools for processing of high-dimensional cytometry data
biocViews: Software,CellBasedAssays,DataRepresentation,ImmunoOncology,
        FlowCytometry,SingleCell,Visualization,Normalization,DataImport
Authors@R: person('Jakob', 'Theorell', email = "jakob.theorell@ki.se",
        role = c("aut", "cre"))
Description: This package is intended to fill the role of conventional cytometry
        pre-processing software, for spectral decomposition, transformation, 
        visualization and cleanup, and to aid further downstream analyses, such
        as with DepecheR, by enabling transformation of flowFrames and flowSets
        to dataframes. Functions for flowCore-compliant automatic 
        1D-gating/filtering are in the pipe line. 
        The package name has been chosen both as it will deal with spectral
        cytometry and as it will hopefully give the user a nice pair of
        spectacles through which to view their data.
BugReports: https://github.com/jtheorell/flowSpecs/issues
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: false
RoxygenNote: 7.2.3
Depends: R (>= 4.0)
Imports: ggplot2 (>= 3.1.0), BiocGenerics (>= 0.30.0), BiocParallel (>=
        1.18.1), Biobase (>= 2.48.0), reshape2 (>= 1.4.3), flowCore (>=
        1.50.0), zoo (>= 1.8.6), stats (>= 3.6.0), methods (>= 3.6.0)
Suggests: testthat, knitr, rmarkdown, BiocStyle, DepecheR
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/flowSpecs
git_branch: RELEASE_3_22
git_last_commit: 2da6fa3
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 04:00:17 UTC; biocbuild
Author: Jakob Theorell [aut, cre]
Maintainer: Jakob Theorell <jakob.theorell@ki.se>
