# Generated by roxygen2: do not edit by hand

export(arcTrans)
export(corrMatCreate)
export(correctUnmix)
export(flowSet2LongDf)
export(oneVsAllPlot)
export(peakNorm)
export(specMatCalc)
export(specUnmix)
importFrom(Biobase,"pData<-")
importFrom(Biobase,pData)
importFrom(BiocGenerics,colnames)
importFrom(BiocGenerics,ncol)
importFrom(BiocGenerics,nrow)
importFrom(BiocParallel,bplapply)
importFrom(flowCore,"exprs<-")
importFrom(flowCore,"parameters<-")
importFrom(flowCore,"sampleNames<-")
importFrom(flowCore,exprs)
importFrom(flowCore,flowSet)
importFrom(flowCore,fsApply)
importFrom(flowCore,keyword)
importFrom(flowCore,pData)
importFrom(flowCore,parameters)
importFrom(flowCore,sampleNames)
importFrom(ggplot2,aes)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_line)
importFrom(ggplot2,facet_wrap)
importFrom(ggplot2,geom_hex)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggsave)
importFrom(ggplot2,scale_fill_gradientn)
importFrom(ggplot2,theme)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(methods,setGeneric)
importFrom(methods,setMethod)
importFrom(reshape2,melt)
importFrom(stats,density)
importFrom(stats,lsfit)
importFrom(stats,mad)
importFrom(stats,median)
importFrom(stats,quantile)
importFrom(stats,var)
importFrom(zoo,rollmean)
