#' A fully stained spectral cytometry sample
#'
#' This is a flowFrame with a PBMC sample stained with 12 fluorochromes.
#' Data acquired on a 44 detector, 3 laser Cytek Aurora® instrument by
#' J Theorell.
#' Date: 2018-10-25.
#'
#' @docType data
#'
#' @usage data(fullPanel)
#'
#' @format An object of class \code{"flowFrame"}
#'
#' @keywords datasets
"fullPanel"
