#' Spectral unmixing matrix
#'
#' This matrix is generated using the specMatCalc function and the unmixCtrls
#' example file.
#'
#' @docType data
#'
#' @usage data(specMat)
#'
#' @format An object of class \code{"matrix"};
#'
#' @keywords datasets
"specMat"
